import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import loggerOptions from '../logger.options';
import { level } from '../level.enum';
import {
  AUTH_CREATED_CRIT_MESSAGE_HANDLER,
  AUTH_DELETED_CRIT_MESSAGE_HANDLER,
  AUTH_HEALTH_CRIT_MESSAGE_HANDLER,
  AUTH_PASSWORD_UPDATED_WARNING_MESSAGE_HANDLER,
  AUTH_READ_WARNING_MESSAGE_HANDLER,
  AUTH_USERNAME_CREATED_WARNING_MESSAGE_HANDLER,
  AUTH_USERNAME_DELETED_WARNING_MESSAGE_HANDLER,
  AUTH_USERNAME_READ_WARNING_MESSAGE_HANDLER,
  AUTH_USERNAME_UPDATED_WARNING_MESSAGE_HANDLER,
} from '@src/app.constants';

export class AuthMessageHandler {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    name: AUTH_CREATED_CRIT_MESSAGE_HANDLER,
  })
  public async authCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.crit, 'critical', 'created'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_DELETED_CRIT_MESSAGE_HANDLER,
  })
  public async authDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.crit, 'critical', 'deleted'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_READ_WARNING_MESSAGE_HANDLER,
  })
  public async authReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'read'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_USERNAME_CREATED_WARNING_MESSAGE_HANDLER,
  })
  public async authUsernameCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-created'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_USERNAME_DELETED_WARNING_MESSAGE_HANDLER,
  })
  public async authUsernameDeletedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-deleted'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_USERNAME_READ_WARNING_MESSAGE_HANDLER,
  })
  public async authUsernameReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_PASSWORD_UPDATED_WARNING_MESSAGE_HANDLER,
  })
  public async authPasswordUpdatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'password-updated'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_USERNAME_UPDATED_WARNING_MESSAGE_HANDLER,
  })
  public async authUsernameUpdatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-updated'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AUTH_HEALTH_CRIT_MESSAGE_HANDLER,
  })
  public async authHealthCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.crit, 'critical', 'health'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
